@@ -1,7 +1,4 @@
 
-/* Avoid warning about empty compilation unit. */
-void keyring_macos_dummy() { }
-
 #ifdef __APPLE__
 
 #include <CoreFoundation/CoreFoundation.h>
@@ -464,29 +461,61 @@ SEXP keyring_macos_is_locked_keyring(SEXP keyring) {
   return ScalarLogical(! (kstatus & kSecUnlockStateStatus));
 }
 
-static const R_CallMethodDef callMethods[]  = {
-  { "keyring_macos_get",    (DL_FUNC) &keyring_macos_get,            3 },
-  { "keyring_macos_set",    (DL_FUNC) &keyring_macos_set,            4 },
-  { "keyring_macos_delete", (DL_FUNC) &keyring_macos_delete,         3 },
-  { "keyring_macos_list",   (DL_FUNC) &keyring_macos_list,           2 },
-  { "keyring_macos_create", (DL_FUNC) &keyring_macos_create,         2 },
-  { "keyring_macos_list_keyring",
-                            (DL_FUNC) &keyring_macos_list_keyring,   0 },
-  { "keyring_macos_delete_keyring",
-                            (DL_FUNC) &keyring_macos_delete_keyring, 1 },
-  { "keyring_macos_lock_keyring",
-                            (DL_FUNC) &keyring_macos_lock_keyring,   1 },
-  { "keyring_macos_unlock_keyring",
-                            (DL_FUNC) &keyring_macos_unlock_keyring, 2 },
-  { "keyring_macos_is_locked_keyring",
-                            (DL_FUNC) &keyring_macos_is_locked_keyring, 1 },
-  { NULL, NULL, 0 }
-};
+#else
 
-void R_init_keyring(DllInfo *dll) {
-  R_registerRoutines(dll, NULL, callMethods, NULL, NULL);
-  R_useDynamicSymbols(dll, FALSE);
-  R_forceSymbols(dll, TRUE);
+#include <R.h>
+#include <R_ext/Rdynload.h>
+#include <Rinternals.h>
+
+SEXP keyring_macos_get(SEXP keyring, SEXP service, SEXP username) {
+  error("only works on macOS");
+  return R_NilValue;
+}
+
+SEXP keyring_macos_set(SEXP keyring, SEXP service, SEXP username,
+                       SEXP password) {
+  error("only works on macOS");
+  return R_NilValue;
+}
+
+SEXP keyring_macos_delete(SEXP keyring, SEXP service, SEXP username) {
+  error("only works on macOS");
+  return R_NilValue;
+}
+
+SEXP keyring_macos_list(SEXP keyring, SEXP service) {
+  error("only works on macOS");
+  return R_NilValue;
+}
+
+SEXP keyring_macos_create(SEXP keyring, SEXP password) {
+  error("only works on macOS");
+  return R_NilValue;
+}
+
+SEXP keyring_macos_list_keyring() {
+  error("only works on macOS");
+  return R_NilValue;
+}
+
+SEXP keyring_macos_delete_keyring(SEXP keyring) {
+  error("only works on macOS");
+  return R_NilValue;
+}
+
+SEXP keyring_macos_lock_keyring(SEXP keyring) {
+  error("only works on macOS");
+  return R_NilValue;
+}
+
+SEXP keyring_macos_unlock_keyring(SEXP keyring, SEXP password) {
+  error("only works on macOS");
+  return R_NilValue;
+}
+
+SEXP keyring_macos_is_locked_keyring(SEXP keyring) {
+  error("only works on macOS");
+  return R_NilValue;
 }
 
 #endif // __APPLE__
